

#include <qapplication.h>
#include <qboxlayout.h>

#include <qmainwindow.h>
#include <qgraphicslayout.h>

#include "sensorpanel.h"
#include "qlchart.h"

QLineChart::QLineChart ( QWidget *parent ): QWidget( parent )
{
    struct T_ValData valData;
    
    setParent( parent );

    qlsSeries1 = qlsSeries2 = qlsSeries3 = qlsSeries4 = nullptr;
    
    valChart.init();
    
    tickYAxis = 5;
    tickXAxis = 5;
    qrXAxisStep = 1.0;


    qcvView = new QChartView ( &qcChart, this );    
    qcvView->setDisabled( true );
//    qcvView->setParent( this );
    qcvView->setRenderHint( QPainter::Antialiasing );

    qcChart.addAxis( &qvaXAxis, Qt::AlignBottom );
    qcChart.addAxis( &qvaYAxis, Qt::AlignLeft );

/*  qlsSeries1 = new QLineSeries();
    qcChart.addSeries( qlsSeries1 ); */

//    qlsSeries->append( 5, 1 );
/*  qlsSeries1->setName( qsFL1 ( "Line 1" ) );
    qlsSeries1->attachAxis( &qvaXAxis );
    qlsSeries1->attachAxis( &qvaYAxis ); */
//    QPen green(Qt::red);
//    green.setWidth(3);
//    qlsSeries->setPen(green);

//    qvaAxisX.setLabelsFont( qfLabel );
    qvaXAxis.setTickCount( tickXAxis );
    qvaXAxis.setRange( 0, 10 );
    qvaXAxis.setReverse ( true );

//  qvaAxisY.setLabelsFont( qfLabel );
    qvaYAxis.setRange ( -5, 1 );

    qcChart.setAnimationOptions( QChart::NoAnimation );
//    qcChart.setTitle( qsFL1 ("Dynamic line") );
    qcChart.legend()->hide();
    qcChart.setMargins( QMargins( 0, 0, 0, 0 ) );
    qcChart.layout()->setContentsMargins( 0, 0, 0, 0 );
    qcChart.setBackgroundVisible( false );


    qlValue1 = new QLabel ( qcvView );
    qlValue2 = new QLabel ( qcvView );
    qlValue3 = new QLabel ( qcvView );
    qlValue4 = new QLabel ( qcvView );
    qlDescr1 = new QLabel ( qcvView );
    qlDescr2 = new QLabel ( qcvView );

    adjustSize();
    
    
    
//    updateDescr();
    valData.init();
    
    valData.visibleValue1 = true;
    valData.visibleValue2 = valData.visibleValue3 = valData.visibleValue4 = false;
    valData.visibleDescr1 = valData.visibleDescr2 = true;
    valData.qsValue1 = qsFL1 ( "val. 1" );
    valData.qsValue2 = qsFL1 ( "val. 2" );
    valData.qsValue3 = qsFL1 ( "val. 3" );
    valData.qsValue4 = qsFL1 ( "val. 4" );
    valData.qsDescr1 = qsFL1 ( "descr. 1" );
    valData.qsDescr2 = qsFL1 ( "descr. 2" );

    setValue( valData );

    chartData1.init();
    chartData2.init();
    chartData3.init();
    chartData4.init();
}

QLineChart::~QLineChart()
{
}

/***************************************************************************
    set current values and descriptions
 ***************************************************************************/

void QLineChart::setValue( struct T_ValData &valData )
{
    QPalette qpValue;
    
    if ( valData.visibleValue1 != valChart.visibleValue1 )
        qlValue1->setVisible( valData.visibleValue1 );
    if ( valData.visibleValue2 != valChart.visibleValue2 )
        qlValue2->setVisible( valData.visibleValue2 );
    if ( valData.visibleValue3 != valChart.visibleValue3 )
        qlValue3->setVisible( valData.visibleValue3 );
    if ( valData.visibleValue4 != valChart.visibleValue4 )
    {
        if ( valData.visibleValue4 )
        {
            qpValue.setColor( QPalette::Disabled, QPalette::WindowText, SensorPanel::color1 );
            qlValue1->setPalette( qpValue );
            qpValue.setColor( QPalette::Disabled, QPalette::WindowText, SensorPanel::color2 );
            qlValue2->setPalette( qpValue );
            qpValue.setColor( QPalette::Disabled, QPalette::WindowText, SensorPanel::color3 );
            qlValue3->setPalette( qpValue );
            qpValue.setColor( QPalette::Disabled, QPalette::WindowText, SensorPanel::color4 );
            qlValue4->setPalette( qpValue );
        }

        qlValue4->setVisible( valData.visibleValue4 );
    }
    if ( valData.visibleDescr1 != valChart.visibleDescr1 )
        qlDescr1->setVisible( valData.visibleDescr1 );
    if ( valData.visibleDescr2 != valChart.visibleDescr2 )
        qlDescr2->setVisible( valData.visibleDescr2 );


    if ( /* valData.qsValue1 != qsFL1 ( "" ) && */ valData.visibleValue1 && valData.qsValue1 != valChart.qsValue1 )
        qlValue1->setText( valData.qsValue1 );
    if ( /* valData.qsValue2 != qsFL1 ( "" ) && */ valData.visibleValue3 && valData.qsValue2 != valChart.qsValue2 )
        qlValue2->setText( valData.qsValue2 );
    if ( /* valData.qsValue3 != qsFL1 ( "" ) && */ valData.visibleValue3 && valData.qsValue3 != valChart.qsValue3 )
        qlValue3->setText( valData.qsValue3 );
    if ( /* valData.qsValue4 != qsFL1 ( "" ) && */ valData.visibleValue4 && valData.qsValue4 != valChart.qsValue4 )
        qlValue4->setText( valData.qsValue4 );
    if ( /* valData.qsDescr1 != qsFL1 ( "" ) && */ valData.visibleDescr1 && valData.qsDescr1 != valChart.qsDescr1 )
        qlDescr1->setText( valData.qsDescr1 );
    if ( /* valData.qsDescr2 != qsFL1 ( "" ) && */ valData.visibleDescr2 && valData.qsDescr2 != valChart.qsDescr2 )
        qlDescr2->setText( valData.qsDescr2 );


    valChart = valData;
}

/***************************************************************************
    add chart value
 ***************************************************************************/

void QLineChart::addValue( struct T_ValData &valData, qreal qrXAxisStepIn )
{
    if ( valData.visibleValue1 )
    {
        if ( !qlsSeries1 )
        {
            qlsSeries1 = new QLineSeries();
            qcChart.addSeries( qlsSeries1 );
            qlsSeries1->setName( qsFL1 ( "Line 1" ) );
            qlsSeries1->attachAxis( &qvaXAxis );
            qlsSeries1->attachAxis( &qvaYAxis );
        }

        addValue ( chartData1, qlsSeries1, valData.qsValue1, qrXAxisStepIn );
    }

    if ( valData.visibleValue2 )
    {
        if ( !qlsSeries2 )
        {
            qlsSeries2 = new QLineSeries();
            qcChart.addSeries( qlsSeries2 );
            qlsSeries2->setName( qsFL1 ( "Line 2" ) );
            qlsSeries2->attachAxis( &qvaXAxis );
            qlsSeries2->attachAxis( &qvaYAxis );
        }

        addValue ( chartData2, qlsSeries2, valData.qsValue2, qrXAxisStepIn );
    }
    else if ( qlsSeries2 )
    {
        qcChart.removeSeries( qlsSeries2 );
        qlsSeries2 = nullptr;
    }

    if ( valData.visibleValue3 )
    {
        if ( !qlsSeries3 )
        {
            qlsSeries3 = new QLineSeries();
            qcChart.addSeries( qlsSeries3 );
            qlsSeries3->setName( qsFL1 ( "Line 3" ) );
            qlsSeries3->attachAxis( &qvaXAxis );
            qlsSeries3->attachAxis( &qvaYAxis );
        }

        addValue ( chartData3, qlsSeries3, valData.qsValue3, qrXAxisStepIn );
    }
    else if ( qlsSeries3 )
    {
        qcChart.removeSeries( qlsSeries3 );
        qlsSeries3 = nullptr;
    }

    if ( valData.visibleValue4 )
    {
        if ( !qlsSeries4 )
        {
            qlsSeries4 = new QLineSeries();
            qcChart.addSeries( qlsSeries4 );
            qlsSeries4->setName( qsFL1 ( "Line 4" ) );
            qlsSeries4->attachAxis( &qvaXAxis );
            qlsSeries4->attachAxis( &qvaYAxis );
            if ( valData.visibleValue4 )
            {
                qlsSeries1->setPen ( SensorPanel::color1 );
                qlsSeries2->setPen ( SensorPanel::color2 );                
                qlsSeries3->setPen ( SensorPanel::color3 );
                qlsSeries4->setPen ( SensorPanel::color4 );
            }
        }

        addValue ( chartData4, qlsSeries4, valData.qsValue4, qrXAxisStepIn );
    }
    else if ( qlsSeries4 )
    {
        qcChart.removeSeries( qlsSeries4 );
        qlsSeries4 = nullptr;
    }    
} // void QLineChart::addValue( struct T_ValData &valData, qreal qrXAxisStepIn )

void QLineChart::addValue ( struct T_ChartData &charData, QLineSeries *qlsSeries, QString qsYAxisValueIn, qreal qrXAxisStepIn )
{
    bool enabOver, enabAdd, enabFirst;

    qreal minXAxis, maxXAxis;

    QPointF qpfPoint;

    QList<QPointF>::reverse_iterator riQlPoint, riQlSuccPoint;

    minXAxis = qvaXAxis.min();
    maxXAxis = qvaXAxis.max(); 

    charData.qrMinValue = qrealMAX;
    charData.qrMaxValue = qrealMIN;

    enabOver = enabAdd = false;
    enabFirst = true;

    riQlSuccPoint = charData.dataList.rend();

    for ( riQlPoint = charData.dataList.rbegin(); !enabOver || riQlPoint != charData.dataList.rend(); !enabOver ? riQlPoint++ : riQlPoint )
    {
        enabOver = riQlPoint == charData.dataList.rend();

        if ( enabFirst )
        {
            if ( riQlPoint != charData.dataList.rend() )
                qpfPoint = *riQlPoint;

            if ( qrealCompare( qpfPoint.x(), maxXAxis  ) < 0 )
                enabAdd = true;

            enabFirst = false;
        }

        if ( !enabOver )
        {
            qpfPoint = *riQlPoint;

            qpfPoint.setX ( qpfPoint.x() + qrXAxisStep );

            if ( enabAdd )
            {
                *riQlPoint = qpfPoint;

                setMinMaxValue ( charData, nullptr, &qpfPoint );
            }
            else
            {
                if ( riQlSuccPoint != charData.dataList.rend() )
                {
                    *riQlSuccPoint = qpfPoint;

                    setMinMaxValue ( charData, nullptr, &qpfPoint );
                }

                riQlSuccPoint = riQlPoint;
            }
        }
    }

    if ( enabAdd )
        charData.dataList.prepend( qpfPoint );

    if ( !charData.dataList.isEmpty() )
    {
        qpfPoint.setX ( minXAxis );
        qpfPoint.setY ( qsYAxisValueIn.toDouble() );
        charData.dataList.first() = qpfPoint;

        setMinMaxValue ( charData, nullptr, &qpfPoint );
    }


    setMinMaxValue ( charData, &qvaYAxis, nullptr );
    updateSeries( charData, qlsSeries );


    if ( qrealCompare ( qrXAxisStepIn, qrXAxisStep ) )
    {
        qrXAxisStep = qrXAxisStepIn;

        update();
    }

} // void addValue ( QLineSeries *qlsSeries, QString qsYAxisValueIn, qreal qrXAxisStepIn )

/***************************************************************************
    update chart series
 ***************************************************************************/

void QLineChart::updateSeries( struct T_ChartData &charData, QAbstractSeries *series )
{    
    QXYSeries *qxySeries;
    
    if ( series )
    {
        qxySeries = static_cast<QXYSeries *>( series );

        qxySeries->replace( charData.dataList );
    }
}

/***************************************************************************
    set the y-axis range
 ***************************************************************************/

void QLineChart::setMinMaxValue( struct T_ChartData &charData, QValueAxis *qvaValue,  QPointF *qpfValue )
{
    qreal qrAxisMax, qrAxisMin, qrAxisMaxRif, qrAxisMinRif, qrAxisMaxNew, qrAxisMinNew;
    qreal qrRifMaxMax, qrRifMaxMin, qrRifMinMax, qrRifMinMin;
    qreal qrAxisRange, qrMaxStep, qrMinStep, qrAutoStep;

    qrAutoStep = 0.05;
 
    if ( qpfValue )
    {
        if ( qrealCompare( qpfValue->y(), charData.qrMaxValue  ) > 0 )
            charData.qrMaxValue = qpfValue->y();

        if ( qrealCompare( qpfValue->y(), charData.qrMinValue  ) < 0 )
            charData.qrMinValue = qpfValue->y();
    }

    if ( qvaValue )
    {
        qrAxisMaxNew = qrAxisMaxRif = qrAxisMax = qvaValue->max();
        qrAxisMinNew = qrAxisMinRif = qrAxisMin = qvaValue->min();
        qrAxisRange = qrAxisMax - qrAxisMin;


        if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        {
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( charData.qrMaxValue * qrAutoStep ) ) );
            qrRifMaxMin = charData.qrMaxValue + abs ( floor ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMin ) > 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue *  qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange *  qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) < 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }
        } // if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        else
        {
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrAutoStep ) );
            qrRifMaxMax = charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMax ) < 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( charData.qrMaxValue * qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( charData.qrMaxValue + abs ( ceil ( qrAxisRange * qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) > 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }

            if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
                qvaValue->setMax( qrAxisMaxNew );
        } // else ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )


        qrAxisMaxNew = qrAxisMaxRif;
        if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
            qvaValue->setMax( qrAxisMaxNew );
        


        if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        {
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( charData.qrMinValue * qrAutoStep ) ) );
            qrRifMinMin = charData.qrMinValue - abs ( ceil ( ( qrAxisRange * qrAutoStep ) ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMin ) < 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( charData.qrMinValue *  qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs (floor ( qrAxisRange *  qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) > 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }
        } // if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        else
        {
            qrRifMinMax = charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrAutoStep ) );
            qrRifMinMax = charData.qrMinValue - abs ( floor ( qrAxisRange * qrAutoStep ) );

            if ( qrealCompare( qrAxisMinRif, qrRifMinMax ) > 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( charData.qrMinValue * qrMinStep ) ) );
                qrAxisMinNew = floor ( charData.qrMinValue - abs ( floor ( qrAxisRange * qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) < 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }

            if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
                qvaValue->setMin( qrAxisMinNew );
        } // else ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )


        qrAxisMinNew = qrAxisMinRif;
        if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
            qvaValue->setMin( qrAxisMinNew );



        charData.qrOldMaxValue = charData.qrMaxValue;
        charData.qrOldMinValue = charData.qrMinValue;

    } // if ( qvaValue )
} // void setMinMaxValue( QValueAxis *qvaValue,  QPointF *qpfValue, struct T_CharData &charData )

/***************************************************************************
    update chart
 ***************************************************************************/

void QLineChart::update()
{
    int sizeFontWidth, sizeFontHeight, sizeFontChart;
    int sizeFontYAxis, sizeFontXAxis, sizeFontValue1;

    qreal sizePercWidth, sizePercHeight;
    qreal maxAxis;
    qreal percPos;

    QSize qsLChart;
    QRect qrValue, qrDescr;
    QFont qfYAxis, qfXAxis, qfValue, qfDescr;

    qsLChart = size();


//    QRectF qrfPlotArea = qcChart.plotArea(); 
//    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << qsLChart.width() << qsLChart.height() <<"\033[0m";


    sizePercWidth = qsLChart.width() / 480.0;
    sizePercHeight = qsLChart.height() / 480.0;

    sizeFontValue1 = 7;
    sizeFontWidth = 5 + ( sizePercWidth * 24 );
    sizeFontHeight = 5 + ( sizePercHeight  * 24 );
    sizeFontChart = ( sizeFontWidth < sizeFontHeight ) ? sizeFontWidth : sizeFontHeight;
    if ( sizeFontChart > sizeFontValue1 )
        sizeFontValue1 = sizeFontChart;

    sizeFontYAxis = ( sizeFontValue1 >= 11 ) ? ( sizeFontValue1 - 4 ) : 7;
    sizeFontXAxis = sizeFontYAxis;


//  qfDescr.setFamily();
    qfDescr.setPointSize( ( sizeFontValue1 >= 11 ) ? ( sizeFontValue1 - 1 ) : 10 );    
    qfDescr.setBold( false );
    qfDescr.setWeight( QFont::Light );
    qfDescr.setStretch( QFont::Condensed );


    tickYAxis = 5;
    tickXAxis = 5;

    maxAxis = 150.0 * qrXAxisStep  * sizePercWidth;
    maxAxis = qRound ( maxAxis / 10.0 ) * 10.0;

    if ( qrealCompare( sizePercHeight, 0.25 ) < 0 )
       tickYAxis = 3;

    if ( qrealCompare( sizePercWidth, 0.25 ) < 0 )
    {
        tickXAxis = 3;
        
        qfDescr.setPointSize( 7 );
        qfDescr.setStretch( QFont::UltraCondensed );
    }

    qfYAxis.setPointSize( sizeFontYAxis );
    qfYAxis.setBold( false );

    qfXAxis = qfYAxis;
    qfXAxis.setPointSize( sizeFontXAxis );

//  qfValue1.setFamily();
    qfValue.setPointSize( sizeFontValue1 );
    qfValue.setBold( false );

    qvaYAxis.setTickCount( tickYAxis );
    qvaYAxis.setLabelsFont( qfYAxis );

    qvaXAxis.setTickCount( tickXAxis );
    qvaXAxis.setRange( 0, maxAxis);
    qvaXAxis.setLabelsFont( qfXAxis );


    percPos = 2.0;
    if ( valChart.visibleValue4 )
        percPos = 10.0;

    qlValue1->setFont( qfValue );
    qlValue1->adjustSize();
    qlValue1->move( 0, ( ( qsLChart.height() - qlValue1->height() ) / percPos ) );
    qlValue1->setFixedWidth( qsLChart.width() );
    qlValue1->setAlignment( Qt::AlignHCenter );

    if ( valChart.visibleValue4 )
    {        
        qrValue = qlValue1->geometry();

        qlValue2->setFont( qfValue );
        qlValue2->adjustSize();
        qlValue2->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue2->setFixedWidth( qsLChart.width() );
        qlValue2->setAlignment( Qt::AlignHCenter );

        qrValue = qlValue2->geometry();

        qlValue3->setFont( qfValue );
        qlValue3->adjustSize();
        qlValue3->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue3->setFixedWidth( qsLChart.width() );
        qlValue3->setAlignment( Qt::AlignHCenter );

        qrValue = qlValue3->geometry();

        qlValue4->setFont( qfValue );
        qlValue4->adjustSize();
        qlValue4->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue4->setFixedWidth( qsLChart.width() );
        qlValue4->setAlignment( Qt::AlignHCenter );
    }

    qlDescr1->setFont( qfDescr );
    qlDescr1->adjustSize();
    qlDescr1->move( 3, ( ( qsLChart.height() - qlDescr1->height() ) / 10.0 ) );
    qlDescr1->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr1->setAlignment( Qt::AlignHCenter );

    qrDescr = qlDescr1->geometry();

    qlDescr2->setFont( qfDescr );
    qlDescr2->adjustSize();
    qlDescr2->move( ( qrDescr.x() + qrDescr.width() ), ( ( qsLChart.height() - qlDescr2->height() ) / 10.0 ) );
    qlDescr2->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr2->setAlignment( Qt::AlignHCenter );
}

/***************************************************************************
    widget resize event
 ***************************************************************************/

void QLineChart::resizeEvent ( QResizeEvent *qreEvent )
{
    Q_UNUSED(qreEvent);

    qcvView->setGeometry( 0, 0, width(), height() );
//    qcChart.setGeometry( -10, 0, width(), height() );

    update();
}


/***************************************************************************
    test chart
 ***************************************************************************/

testChart::testChart() : QWidget()
{
    struct QLineChart::T_ValData valData;

    setWindowTitle ( qsFL1 ( "Chart (only for test)" ) );

    QPalette palette3, palette4;
    QBrush brush3, brush4;

    brush3 = QBrush (QColor(255, 255, 255, 255));
    brush3.setStyle(Qt::SolidPattern);

    palette3.setBrush(QPalette::Active, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Inactive, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Base, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Window, brush3);

    brush4 = brush3;
    brush4.setColor(QColor(200, 57, 241, 255));

    palette4 = palette3;
    palette4.setBrush(QPalette::Active, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Inactive, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Base, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Window, brush4);

    qwChart = new QWidget(this);
    qwChart->setObjectName("qwChart");
    qwChart->setGeometry(QRect(0, 0, 96, 96));

    qfChart = new QFrame(qwChart);
    qfChart->setObjectName("qfChart");
//        qfChart->setGeometry(QRect(0, 0, 250, 250));
    qfChart->resize(qwChart->size());

    qfChart->setPalette(palette3);
    qfChart->setAutoFillBackground(true);
    qfChart->setFrameShape(QFrame::Shape::Panel);
    qfChart->setFrameShadow(QFrame::Shadow::Raised);

    qwChart2 = new QWidget(this);
    qwChart2->setObjectName("qwChart2");
    qwChart2->setGeometry(QRect(300, 0, 250, 250));

    qfChart2 = new QFrame(qwChart2);
    qfChart2->setObjectName("qfChart2");
//        qfChart2->setGeometry(QRect(0, 0, 250, 250));
    qfChart2->resize(qwChart2->size());

//        qfChart2->setPalette(palette4);
    qfChart2->setAutoFillBackground(true);
    qfChart2->setFrameShape(QFrame::Shape::Panel);
    qfChart2->setFrameShadow(QFrame::Shadow::Raised);


    qclChart = new QLineChart (qfChart);
    qclChart->setGeometry( 0, 0, 96, 96 );


//    QRectF qrfPlotArea = qclChart->qcChart.plotArea(); 
//    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << size().width() << size().height() <<"\033[0m";


    qclChart2 = new QLineChart (qfChart2);
    qclChart2->setGeometry( 0, 0, 250, 250 );


//    show();

    resize(600, 600);

    testStepY = 0.1;
    testStepX = 0.1;
    testPosY = 0;

    valData.init();
    
    valData.visibleValue1 = true;
    valData.visibleValue2 = valData.visibleValue3 = valData.visibleValue4 = false;
    valData.visibleDescr1 = valData.visibleDescr2 = true;
    valData.qsValue1 = qsFL1 ( "val. 1" );
    valData.qsValue2 = qsFL1 ( "val. 2" );
    valData.qsValue3 = qsFL1 ( "val. 3" );
    valData.qsValue4 = qsFL1 ( "val. 4" );
    valData.qsDescr1 = qsFL1 ( "descr. 1" );
    valData.qsDescr2 = qsFL1 ( "descr. 2" );

    valData.qsValue1 = QString::number ( testPosY );
    
    qclChart2->setValue( valData );
//  qclChart->addData( testPosY );
    qclChart2->addValue( valData, testStepX );



    idTimer = startTimer( 100 );

//   setCentralWidget(qwChart3);

/*   QMainWindow * win = (QMainWindow *) QApplication::activeWindow();        

     foreach (QWidget *w, qApp->topLevelWidgets())
        if (win = qobject_cast<QMainWindow*>(w))
          break; */
}

testChart::~testChart()
{

}

void testChart::timerEvent( QTimerEvent * )
{
    struct QLineChart::T_ValData valData;

    static int enabKill = 0;

    if ( testPosY == 0.0 )
        testPosY = 2.0;

    testPosY += testStepY;

    if ( enabKill >= 0 && enabKill < 10 )
    {
        if ( testPosY > 3 )
        {        
            testPosY = -20.0;

            if ( enabKill == 1 )
                enabKill = 9;

            enabKill++;
        }
    }
    else if ( enabKill >= 10 && enabKill < 12 )
    {
        if ( testPosY > 30.0 )
        {
            testPosY = 9.0;
            enabKill++;
        }
    }
    else if ( enabKill < 15 )
    {
        if ( testPosY > 12.0 )
        {
            testPosY = 9.0;

            enabKill++;
        }
    }


    valData.init();
    
    valData.visibleValue1 = true;
    valData.visibleValue2 = valData.visibleValue3 = valData.visibleValue4 = false;
    valData.visibleDescr1 = valData.visibleDescr2 = true;
    valData.qsValue1 = qsFL1( "TCTL %1 C" ).arg ( testPosY, 0, 'f', 0, QChar::fromLatin1( ' ' ) );
    valData.qsDescr1 = qsFL1 ( "descr. 1" );
    valData.qsDescr2 = qsFL1 ( "descr. 2" );

    valData.qsValue1 = QString::number ( testPosY );
    
    qclChart2->setValue( valData );
//    qclChart->addData( posY );
    qclChart2->addValue( valData, testStepX );
}
