#ifndef CHART_H
#define CHART_H

#include <qstring.h>
#include <qlabel.h>

#include <qmainwindow.h>
#include <qwidget.h>

#include <qchart.h>
#include <qchartview.h>
#include <QtCharts/QAbstractAxis>
#include <QtCharts/QSplineSeries>
#include <QtCharts/QValueAxis>
#include <QtCore/QRandomGenerator>
#include <QtCore/QTimer>

#include "general.h"


#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
using namespace QtCharts;
#endif

class Chart;

class QLineChart : public QWidget
{
    Q_OBJECT
public:

    struct T_ChartData
    {
        qreal qrOldMaxValue;
        qreal qrOldMinValue;
        qreal qrMaxValue;
        qreal qrMinValue;

        QList<QPointF> dataList;

        void init()
        {
            qrOldMaxValue = qrealMIN;
            qrOldMinValue = qrealMAX;
            qrMaxValue = qrealMAX;
            qrMinValue = qrealMIN;

            dataList.clear();
        }
    };

    struct T_ValData
    {
        bool visibleValue1;
        bool visibleValue2;
        bool visibleValue3;
        bool visibleValue4;
        bool visibleDescr1;
        bool visibleDescr2;

        QString qsValue1;
        QString qsValue2;
        QString qsValue3;
        QString qsValue4;
        QString qsDescr1; 
        QString qsDescr2;

        void init()
        {
            visibleValue1 = true;
            visibleValue2 = true;
            visibleValue3 = true;
            visibleValue4 = true;
            visibleDescr1 = true;
            visibleDescr2 = true;

            qsValue1 = qsFL1 ( "" );
            qsValue2 = qsFL1 ( "" );
            qsValue3 = qsFL1 ( "" );
            qsValue4 = qsFL1 ( "" );
            qsDescr1 = qsFL1 ( "" );
            qsDescr2 = qsFL1 ( "" );
        }
    };

    QLineChart ( QWidget *parent );
    ~QLineChart();

    void setValue( struct T_ValData &valData );
    void addValue( struct T_ValData &valData, qreal qrXAxisStepIn );
    void addValue( struct T_ChartData &charData, QLineSeries *qlsSeries, QString qsYAxisValueIn, qreal qrXAxisStepIn );
    void setMinMaxValue( struct T_ChartData &charData, QValueAxis *qvaValue, QPointF *qpfValue );

protected:
    void resizeEvent( QResizeEvent *qreEvent ) override;

private:
    void updateSeries( struct T_ChartData &charData, QAbstractSeries *series );
    void update();

    struct T_ValData valChart;

    QLabel *qlValue1;
    QLabel *qlValue2;
    QLabel *qlValue3;
    QLabel *qlValue4;
    QLabel *qlDescr1;
    QLabel *qlDescr2;

    int tickYAxis;
    int tickXAxis;
    qreal qrXAxisStep;

    QChart qcChart;
    QChartView *qcvView;
    QLineSeries *qlsSeries1;
    QLineSeries *qlsSeries2;
    QLineSeries *qlsSeries3;
    QLineSeries *qlsSeries4;
    QValueAxis qvaXAxis;
    QValueAxis qvaYAxis;

    struct T_ChartData chartData1;
    struct T_ChartData chartData2;
    struct T_ChartData chartData3;
    struct T_ChartData chartData4;
};


class testChart : public QWidget
{
   Q_OBJECT
public:
    testChart();
    ~testChart();
    
    QLineChart *qclChart;
    QLineChart *qclChart2;

    QWidget *qwChart;
    QWidget *qwChart2;
    QFrame *qfChart;
    QFrame *qfChart2;

private:

    void timerEvent( QTimerEvent * ) override;

    int idTimer;
    
    qreal testStepY;
    qreal testStepX;
    qreal testPosY;    
};

#endif
